#include "sampledWaveElevation.H"

Foam::functionObjects::sampledWaveElevation::volFieldSampler::volFieldSampler
(
    const word& interpolationScheme,
    const GeometricField<scalar, fvPatchField, volMesh>& field,
    const PtrList<sampledSet>& samplers
)
:
    List<Field<scalar>>(samplers.size()),
    name_(field.name())
{
    autoPtr<interpolation<scalar>> interpolator
    (
        interpolation<scalar>::New(interpolationScheme, field)
    );

    forAll(samplers, setI)
    {
        Field<scalar>& values = this->operator[](setI);
        const sampledSet& samples = samplers[setI];

        values.setSize(samples.size());
        forAll(samples, sampleI)
        {
            const point& samplePt = samples[sampleI];
            label celli = samples.cells()[sampleI];
            label facei = samples.faces()[sampleI];

            if (celli == -1 && facei == -1)
            {
                // Special condition for illegal sampling points
                values[sampleI] = pTraits<scalar>::max;
            }
            else
            {
                values[sampleI] = interpolator().interpolate
                (
                    samplePt,
                    celli,
                    facei
                );
            }
        }
    }
}


Foam::functionObjects::sampledWaveElevation::volFieldSampler::volFieldSampler
(
    const GeometricField<scalar, fvPatchField, volMesh>& field,
    const PtrList<sampledSet>& samplers
)
:
    List<Field<scalar>>(samplers.size()),
    name_(field.name())
{
    forAll(samplers, setI)
    {
        Field<scalar>& values = this->operator[](setI);
        const sampledSet& samples = samplers[setI];

        values.setSize(samples.size());
        forAll(samples, sampleI)
        {
            label celli = samples.cells()[sampleI];

            if (celli ==-1)
            {
                values[sampleI] = pTraits<scalar>::max;
            }
            else
            {
                values[sampleI] = field[celli];
            }
        }
    }
}


Foam::functionObjects::sampledWaveElevation::volFieldSampler::volFieldSampler
(
    const List<Field<scalar>>& values,
    const word& name
)
:
    List<Field<scalar>>(values),
    name_(name)
{}
